/*-
 * Copyright (c) 2013 Tomasz Walaszek
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <fcntl.h>
#include <kvm.h>

#include <limits.h>
#include <paths.h>

#include <err.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/sysctl.h>
#include <sys/user.h>

int
main(int argc, char *argv[])
{
	char **args, errbuf[_POSIX2_LINE_MAX];
	const char *nlistf, *memf;
	char *end;

	int cnt, i;
	pid_t pid;

	kvm_t *kd;

	struct kinfo_proc *kp;

	if (argc < 2) {
		printf("Usage: pargs <pid>\n");
		exit(1);
	}
	pid = (int)strtol(argv[1], &end, 10);
	if (end == argv[1] || *end != '\0' || errno == ERANGE)
		errx(1, "Invalid process id %s", argv[1]);

	nlistf = NULL;
	memf = _PATH_DEVNULL;

	kd = kvm_openfiles(nlistf, memf, NULL, O_RDONLY, errbuf);
	if (kd == 0)
		errx(1, "%s", errbuf);

	kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &cnt);
	if (kp == NULL)
		errx(1, "Process %d does not exist", pid);

	args = kvm_getargv(kd, kp, 0);

	if (args == NULL)
		errx(1, "Process %d does not have arguments", pid);

	i = 0;
	while (args[i] != NULL) {
		printf("Arg%d  %s\n", i, args[i]);
		i++;
	}

	return 0;
}
