/*-
 * Copyright (c) 2013 Tomasz Walaszek
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <fcntl.h>
#include <kvm.h>

#include <limits.h>
#include <paths.h>

#include <err.h>
#include <errno.h>

#include <sys/param.h>
#include <sys/types.h>
#include <libutil.h>
#include <sys/user.h>
#include <sys/sysctl.h>


int
main(int argc, char *argv[])
{
	char errbuf[_POSIX2_LINE_MAX];
	char *end;
	const char *nlistf, *memf;

	int cnt;
	pid_t pid;

	kvm_t *kd;

	struct kinfo_vmentry *vm;
	struct kinfo_proc *kp;

	if (argc < 2) {
		printf("Usage: pldd <pid>\n");
		return 1;
	}
	pid = (int)strtol(argv[1], &end, 10);
	if (end == argv[1] || *end != '\0' || errno == ERANGE)
		errx(1, "Invalid process id %s", argv[1]);

	nlistf = NULL;
	memf = _PATH_DEVNULL;

	kd = kvm_openfiles(nlistf, memf, NULL, O_RDONLY, errbuf);
	if (kd == 0)
		errx(1, "%s", errbuf);

	kp = kvm_getprocs(kd, KERN_PROC_PID, pid, &cnt);
	if (kp == NULL)
		errx(1, "Process %d does not exists", pid);

	vm = kinfo_getvmmap(pid, &cnt);
	if (vm == NULL)
		errx(1, "Cant get memory map information for process %d", pid);

	printf("%d: %s\n", pid, kp->ki_comm);
	for (int i = 0; i < cnt; i++) {
		if (!i) {
			vm++;
			continue;
		}
		if (vm->kve_type == KVME_TYPE_VNODE && vm->kve_protection & KVME_PROT_EXEC)
			printf("%s\n", vm->kve_path);
		vm++;
	}

	return 0;
}
